@@ -28,6 +28,8 @@
 
 import aux
 from aux import *
+import storage
+from storage import *
 import tensor_wrapper
 from tensor_wrapper import *
 import candecomp
@@ -45,6 +47,7 @@ from SpectralTensorTrain import *
 
 __all__ = []
 __all__ += aux.__all__
+__all__ += storage.__all__
 __all__ += tensor_wrapper.__all__
 __all__ += candecomp.__all__
 __all__ += TensorTrainVec.__all__
