@@ -24,7 +24,7 @@
 
 __all__ = ['add','mul','dot','kron','contraction','norm','sd','cg','bicgstab','gmres']
 
-import sys
+import logging
 import numpy as np
 import numpy.linalg as npla
 from scipy import linalg as scla
@@ -47,155 +47,11 @@ def mul(A,B):
     return A * B
 
 def dot(A,B):
-    from TensorToolbox.core import TTvec
-    from TensorToolbox.core import TTmat
-
-    if isinstance(A,TTvec) and isinstance(B,TTvec) and not isinstance(A,TTmat) and not isinstance(B,TTmat):
-        if not A.init or not B.init: raise NameError("TensorToolbox.multilinalg.dot: TT not initialized correctly")
-        
-        # TT vector-vector dot product
-        # Check consistency
-        if A.shape() != B.shape(): raise NameError("TensorToolbox.multilinalg.dot: A.shape != B.shape")
-        
-        Y = []
-        for i,(Ai,Bi) in enumerate(zip(A.TT,B.TT)):
-            Ai_rsh = np.reshape(Ai,(Ai.shape[0],1,Ai.shape[1],Ai.shape[2]))
-            Ai_rsh = np.transpose(Ai_rsh,axes=(0,3,1,2))
-            Ai_rsh = np.reshape(Ai_rsh,(Ai.shape[0]*Ai.shape[2],Ai.shape[1]))
-            
-            Bi_rsh = np.transpose(Bi,axes=(1,0,2))
-            Bi_rsh = np.reshape(Bi_rsh,(Bi.shape[1],Bi.shape[0]*Bi.shape[2]))
-
-            Yi_rsh = np.dot(Ai_rsh,Bi_rsh)
-            Yi_rsh = np.reshape(Yi_rsh,(Ai.shape[0],Ai.shape[2],1,Bi.shape[0],Bi.shape[2]))
-            Yi_rsh = np.transpose(Yi_rsh,axes=(0,3,2,1,4))
-            Yi = np.reshape(Yi_rsh,(Ai.shape[0]*Bi.shape[0],1,Ai.shape[2]*Bi.shape[2]))
-            
-            if i == 0:
-                out = Yi[:,0,:]
-            else:
-                out = np.dot(out, Yi[:,0,:])
-        
-        return out[0,0]
-    
-    elif isinstance(A,np.ndarray) and isinstance(B,np.ndarray):
+    if isinstance(A,np.ndarray) and isinstance(B,np.ndarray):
         # tensor-tensor dot product (np.tensordot)
         return np.tensordot(A,B,(range(A.ndim),range(B.ndim)))
-    
-    elif isinstance(A,TTmat) and isinstance(B,TTvec) and not isinstance(B,TTmat):
-        
-        if not A.init or not B.init: raise NameError("TensorToolbox.multilinalg.dot: TT not initialized correctly")
-
-        # TT matrix-vector dot product
-        # Check consistency
-        if (A.sparse_only and len(A.sparse_TT) != len(B.TT)) or (not A.sparse_only and len(A.TT) != len(B.TT)):
-            raise NameError("TensorToolbox.multilinalg.dot: A and B must have the same number of cores")
-        
-        for bsize,Acols in zip(B.shape(),A.ncols):
-            if bsize != Acols:
-                raise NameError("TensorToolbox.multilinalg.dot: Matrix and Vector mode dimensions are not consistent")
-        
-        Y = []
-        for i,(Ai,Bi,is_sparse,sp_A) in enumerate(zip(A.TT,B.TT,A.is_sparse,A.sparse_TT)):
-            Bi_rsh = np.transpose(Bi,axes=(1,0,2))
-            Bi_rsh = np.reshape(Bi_rsh,(Bi.shape[1],Bi.shape[0]*Bi.shape[2]))
-            
-            if is_sparse:
-                Yi_rsh = sp_A.dot(Bi_rsh)
-            else:
-                Ai_rsh = np.reshape(Ai,(Ai.shape[0],A.nrows[i],A.ncols[i],Ai.shape[2]))
-                Ai_rsh = np.transpose(Ai_rsh,axes=(0,3,1,2))
-                Ai_rsh = np.reshape(Ai_rsh,(Ai.shape[0]*Ai.shape[2]*A.nrows[i],A.ncols[i]))
-                Yi_rsh = np.dot(Ai_rsh,Bi_rsh)
-            
-            Ai0 = A.ranks()[i]
-            Ai2 = A.ranks()[i+1]
-            Yi_rsh = np.reshape(Yi_rsh,(Ai0,Ai2,A.nrows[i],Bi.shape[0],Bi.shape[2]))
-            Yi_rsh = np.transpose(Yi_rsh,axes=(0,3,2,1,4))
-            Yi = np.reshape(Yi_rsh,(Ai0*Bi.shape[0],A.nrows[i],Ai2*Bi.shape[2]))
-            
-            Y.append(Yi)
-                
-        return TTvec(Y)
-    
-    elif isinstance(A,TTmat) and isinstance(B,TTmat):
-        
-        if not A.init or not B.init: raise NameError("TensorToolbox.multilinalg.dot: TT not initialized correctly")
-
-        # TT matrix-matrix dot product
-        # Check consistency
-        if len(A.TT) != len(B.TT):
-            raise NameError("TensorToolbox.multilinalg.dot: A and B must have the same number of cores")
-        
-        for Brows,Acols in zip(B.nrows,A.ncols):
-            if Brows != Acols:
-                raise NameError("TensorToolbox.multilinalg.dot: Matrices mode dimensions are not consistent")
-        
-        Y = []
-        for i,(Ai,Bi) in enumerate(zip(A.TT,B.TT)):
-            Ai_rsh = np.reshape(Ai,(Ai.shape[0],A.nrows[i],A.ncols[i],Ai.shape[2]))
-            Ai_rsh = np.transpose(Ai_rsh,axes=(0,3,1,2))
-            Ai_rsh = np.reshape(Ai_rsh,(Ai.shape[0]*Ai.shape[2]*A.nrows[i],A.ncols[i]))
-
-            Bi_rsh = np.reshape(Bi,(Bi.shape[0],B.nrows[i],B.ncols[i],Bi.shape[2]))
-            Bi_rsh = np.transpose(Bi_rsh,axes=(1,0,3,2))
-            Bi_rsh = np.reshape(Bi_rsh,(B.nrows[i],Bi.shape[0]*Bi.shape[2]*B.ncols[i]))
-            
-            Yi_rsh = np.dot(Ai_rsh,Bi_rsh)
-            
-            Yi_rsh = np.reshape(Yi_rsh,(Ai.shape[0],Ai.shape[2],A.nrows[i],Bi.shape[0],Bi.shape[2],B.ncols[i]))
-            Yi_rsh = np.transpose(Yi_rsh,axes=(0,3,2,5,1,4))
-            Yi = np.reshape(Yi_rsh,(Ai.shape[0]*Bi.shape[0],A.nrows[i]*B.ncols[i],Ai.shape[2]*Bi.shape[2]))
-            
-            Y.append(Yi)
-        
-        return TTmat(Y,A.nrows,B.ncols)
-    
-    elif isinstance(A,TTmat) and isinstance(B,np.ndarray):
-        
-        if not A.init: raise NameError("TensorToolbox.multilinalg.dot: TT not initialized correctly")
-        
-        # matrix-vector dot product with TTmat and full vector
-        # Check consistency
-        if len(A.shape()) != B.ndim:
-            raise NameError("TensorToolbox.multilinalg.dot: A and B must have the same number of dimensions")
-        for bsize,Acols in zip(B.shape,A.ncols):
-            if bsize != Acols:
-                raise NameError("TensorToolbox.multilinalg.dot: Matrix and Vector mode dimensions are not consistent")
-        
-        Bshape = B.shape
-        Y = np.reshape(B,( (1,) + Bshape ))
-        Yshape = Y.shape
-        
-        for k, Ak in enumerate(A.TT):
-            # Note: Ak(alpha_{k-1},(i_k,j_k),alpha_{k})
-            # Reshape it to Ak((alpha_{k},i_k),(alpha_{k-1},j_k))
-            alpha0 = Ak.shape[0]
-            alpha1 = Ak.shape[2]
-            Ak_rsh = np.reshape(Ak, (alpha0,A.nrows[k],A.ncols[k],alpha1))
-            Ak_rsh = np.transpose(Ak_rsh, axes=(3,1,0,2)) # Ak(alpha_{k},i_k,alpha_{k},j_k)
-            Ak_rsh = np.reshape(Ak_rsh, (alpha1 * A.nrows[k], alpha0 * A.ncols[k]))
-            
-            # Reshape Y to Y((alpha_{k-1},j_k),(i_1,..,i_k-1,j_k+1,..,j_d))
-            Y = np.transpose(Y,axes=(0,k+1)+tuple(range(1,k+1))+tuple(range(k+2,len(Bshape)+1)))
-            Y = np.reshape( Y,(alpha0 * Yshape[k+1], np.prod(Yshape[1:k+1]) * np.prod(Yshape[k+2:])) )
-            
-            # Dot product
-            Y = np.dot(Ak_rsh,Y)
-            
-            # Reshape Y
-            Y = np.reshape(Y, (alpha1, A.nrows[k]) + Yshape[1:k+1] + Yshape[k+2:])
-            Y = np.transpose(Y, axes=(0,) + tuple(range(2,k+2)) + (1,) + tuple(range(k+2,len(Bshape)+1)) )
-            Yshape = Y.shape
-        
-        if Y.shape[0] != 1: raise NameError("TensorToolbox.multilinalg.dot: Last core dimenstion error")
-        
-        Y = np.reshape(Y,Y.shape[1:])
-        
-        return Y
-    
     else:
-        raise NameError("TensorToolbox.multilinalg.dot: Inner product not implemented for the input types")
+        return A.dot(B)
 
 def kron(A,B):
     """
@@ -310,7 +166,7 @@ def norm(A,ord='fro',round_eps=1e-10,eps=1e-4,maxit=1000,pow_guess=None,info=Fal
             q = pow_guess
             lmb = 1.
             k = 0
-            while norm( dot(A,q) - lmb * q ,2) > eps and k < maxit:
+            while norm( dot(A,q) - q * lmb ,2) > eps and k < maxit:
                 k += 1
                 q = dot(A,q).rounding(round_eps)
                 q *= 1./norm(q,2)
@@ -347,6 +203,14 @@ def sd(A,b,x0=None,eps=1e-8,maxit=1000,eps_round=1e-10,ext_info=False):
     
     """
 
+    logger = logging.getLogger(__name__)
+    logger.propagate = False
+    ch = logging.StreamHandler()
+    formatter = logging.Formatter("%(asctime)s %(levelname)s:%(name)s: %(message)s",
+                                  "%Y-%m-%d %H:%M:%S")
+    ch.setFormatter(formatter)
+    logger.addHandler(ch)
+
     from TensorToolbox.core import TTvec
     from TensorToolbox.core import zerosvec
     from TensorToolbox.core import TTmat
@@ -384,19 +248,18 @@ def sd(A,b,x0=None,eps=1e-8,maxit=1000,eps_round=1e-10,ext_info=False):
         delta_0 = res[-1]
         while i < maxit and res[-1] > eps**2. * delta_0:
             alpha = dot(r,r)/dot(p,r)
-            x += alpha * r
+            x += r * alpha
             x.rounding(eps_round)
-            r -= alpha * p
+            r -= p * alpha
             r.rounding(eps_round)
             res.append(norm(r,2))
             p = dot(A,r).rounding(eps_round)
             p.rounding(eps_round)
             
             if isinstance(x,TTvec):
-                sys.stdout.write("SD: err=%e ranks=%s\r" % (res[-1],x.ranks()))
+                logger.info("SD: err=%e ranks=%s" % (res[-1],x.ranks()))
             elif isinstance(x,np.ndarray):
-                sys.stdout.write("SD: err=%e \r" % (res[-1]))
-            sys.stdout.flush()
+                logger.info("SD: err=%e " % (res[-1]))
 
             i += 1
 
@@ -431,6 +294,14 @@ def cg(A,b,x0=None,eps=1e-8,maxit=1000,eps_round=1e-10,ext_info=False):
     
     """
 
+    logger = logging.getLogger(__name__)
+    logger.propagate = False
+    ch = logging.StreamHandler()
+    formatter = logging.Formatter("%(asctime)s %(levelname)s:%(name)s: %(message)s",
+                                  "%Y-%m-%d %H:%M:%S")
+    ch.setFormatter(formatter)
+    logger.addHandler(ch)
+
     from TensorToolbox.core import TTvec
     from TensorToolbox.core import zerosvec
     from TensorToolbox.core import TTmat
@@ -471,13 +342,13 @@ def cg(A,b,x0=None,eps=1e-8,maxit=1000,eps_round=1e-10,ext_info=False):
             q = dot(A,d)
             if is_tt: q.rounding(eps_round)
             alpha = res[-1]/dot(d,q)
-            x += alpha * d
+            x += d * alpha
             if is_tt: x.rounding(eps_round)
             if i % 5 == 0:
                 r = (b - dot(A,x))
                 if is_tt: r.rounding(eps_round)
             else:
-                r -= alpha * q
+                r -= q * alpha
                 if is_tt: r.rounding(eps_round)
             delta_old = res[-1]
             res.append( dot(r,r) )
@@ -487,12 +358,10 @@ def cg(A,b,x0=None,eps=1e-8,maxit=1000,eps_round=1e-10,ext_info=False):
             d += r
             if is_tt: d.rounding(eps_round)
 
-            sys.stdout.write("\033[K")
             if isinstance(x,TTvec):
-                sys.stdout.write("CG: err=%e ranks=%s\r" % (res[-1],x.ranks() ))
+                logger.info("CG: err=%e ranks=%s" % (res[-1],x.ranks() ))
             elif isinstance(x,np.ndarray):
-                sys.stdout.write("CG: err=%e \r" % (res[-1]))
-            sys.stdout.flush()
+                logger.info("CG: err=%e " % (res[-1]))
 
             i += 1
         
@@ -528,6 +397,14 @@ def bicgstab(A,b,x0=None,eps=1e-8,maxit=1000,eps_round=1e-10,ext_info=False):
     
     """
 
+    logger = logging.getLogger(__name__)
+    logger.propagate = False
+    ch = logging.StreamHandler()
+    formatter = logging.Formatter("%(asctime)s %(levelname)s:%(name)s: %(message)s",
+                                  "%Y-%m-%d %H:%M:%S")
+    ch.setFormatter(formatter)
+    logger.addHandler(ch)
+
     from TensorToolbox.core import TTvec
     from TensorToolbox.core import zerosvec
     from TensorToolbox.core import TTmat
@@ -570,7 +447,7 @@ def bicgstab(A,b,x0=None,eps=1e-8,maxit=1000,eps_round=1e-10,ext_info=False):
                 return (x,conv)
 
         beta = (rho_new/rho_old) * (alpha/omega)
-        p = r + beta * (p - omega * v)
+        p = r + (p - v * omega) * beta
         p.rounding(eps_round)
         v = dot(A,p).rounding(eps_round)
         r0v = dot(r0,v)
@@ -587,11 +464,11 @@ def bicgstab(A,b,x0=None,eps=1e-8,maxit=1000,eps_round=1e-10,ext_info=False):
                 return (x,conv)
 
         alpha = rho_new / r0v   
-        s = r - alpha * v
+        s = r - v * alpha
         s.rounding(eps_round)
 
         if norm(s,2) <= eps**2. * delta_0: # Convergence already reached, update and break loop
-            x = x + alpha * p
+            x = x + p * alpha
             conv = True
             if ext_info:
                 info = {'iter': i,
@@ -604,18 +481,17 @@ def bicgstab(A,b,x0=None,eps=1e-8,maxit=1000,eps_round=1e-10,ext_info=False):
 
         t = dot(A,s).rounding(eps_round)
         omega = dot(t,s)/norm(t,2)
-        x = x + alpha * p + omega * s
+        x = x + p * alpha +  s * omega
         x.rounding(eps_round)
         # The exit was here, but we can postpone it ...
-        r = s - omega * t
+        r = s - t * omega
         r.rounding(eps_round)
 
         # Update new -> old
         delta_new = norm(r,2)
         rho_old = rho_new
 
-        sys.stdout.write("Bi-CGSTAB: err=%e\r" % (delta_new))
-        sys.stdout.flush()
+        logger.info("Bi-CGSTAB: err=%e" % (delta_new))
 
     conv = (i < maxit)
     if ext_info:
@@ -651,6 +527,14 @@ def gmres(A,b,x0=None,eps=1e-8,maxit=1000,restart=1000,eps_round=1e-10,ext_info=
     :note: not optimized for symmetric A
     """
 
+    logger = logging.getLogger(__name__)
+    logger.propagate = False
+    ch = logging.StreamHandler()
+    formatter = logging.Formatter("%(asctime)s %(levelname)s:%(name)s: %(message)s",
+                                  "%Y-%m-%d %H:%M:%S")
+    ch.setFormatter(formatter)
+    logger.addHandler(ch)
+
     from TensorToolbox.core import TTvec
     from TensorToolbox.core import zerosvec
     from TensorToolbox.core import TTmat
@@ -664,8 +548,7 @@ def gmres(A,b,x0=None,eps=1e-8,maxit=1000,restart=1000,eps_round=1e-10,ext_info=
     elif not isinstance(x0,TTvec) or b.shape() != x0.shape():
         raise NameError("TensorToolbox.multilinalg.cg: Initial guess must be in TT format and have the same shape of b")
     
-    sys.stdout.write("GMRES: Starting\r")
-    sys.stdout.flush()
+    logger.info("GMRES: Starting")
 
     # Iterate
     x = x0.copy()
@@ -679,7 +562,7 @@ def gmres(A,b,x0=None,eps=1e-8,maxit=1000,restart=1000,eps_round=1e-10,ext_info=
                 y = npla.solve(H[:-1,:],g[:-1])
                 # Compute
                 for i in range(restart):
-                    x += y[i] * v[i]
+                    x += v[i] * y[i]
                     x.rounding(eps_round)
             
             Q = np.eye(restart+1)
@@ -687,19 +570,19 @@ def gmres(A,b,x0=None,eps=1e-8,maxit=1000,restart=1000,eps_round=1e-10,ext_info=
             g = np.zeros((restart+1))
             r = (b - dot(A,x)).rounding(eps_round)
             g[0] = norm(r,2) # beta
-            v = [1./g[0] * r]
+            v = [r * 1./g[0]]
             j = 0
         
         w = dot(A,v[j]).rounding(eps_round)
         for i in range(j+1):
             H[i,j] = dot(w,v[i])
-            w -= H[i,j] * v[i]
+            w -= v[i] * H[i,j]
             w.rounding(eps_round)
         H[j+1,j] = norm(w,2)
         if np.abs(H[j+1,j]) < np.spacing(1):
             # lucky break
             break
-        v.append(1./H[j+1,j] * w)
+        v.append( w * 1./H[j+1,j] )
         
         # Apply old givens rotations to the new column
         H[:j+2,j] = np.dot(Q[:j+2,:j+2], H[:j+2,j])
@@ -719,10 +602,9 @@ def gmres(A,b,x0=None,eps=1e-8,maxit=1000,restart=1000,eps_round=1e-10,ext_info=
         err.append(np.abs(g[j+1]))
 
         if isinstance(x,TTvec):
-            sys.stdout.write("GMRES: err=%e ranks=%s\r" % (err[-1],x.ranks() ))
+            logger.info("GMRES: err=%e ranks=%s" % (err[-1],x.ranks() ))
         elif isinstance(x,np.ndarray):
-            sys.stdout.write("GMRES: err=%e \r" % (err[-1]))
-        sys.stdout.flush()
+            logger.info("GMRES: err=%e" % (err[-1]))
         
         if err[-1] < eps:
             # convergent break
@@ -737,13 +619,12 @@ def gmres(A,b,x0=None,eps=1e-8,maxit=1000,restart=1000,eps_round=1e-10,ext_info=
     y = npla.solve(H[:j+1,:j+1],g[:j+1])
     # Compute solution
     for i in range(j+1):
-        x += y[i] * v[i]
+        x += v[i] * y[i]
         x.rounding(eps_round)
     
     r = (b-dot(A,x)).rounding(eps_round)
 
-    sys.stdout.write("GMRES: Done \r")
-    sys.stdout.flush()
+    logger.info("GMRES: Done ")
     
     if ext_info:
         info = {'iter': counter,
