@@ -26,9 +26,12 @@
 
 import run_unit_tests
 from run_unit_tests import *
+import TestTT
+from TestTT import *
 
 __all__ = []
 __all__ += run_unit_tests.__all__
+__all__ += TestTT.__all__
 
 __author__ = "Daniele Bigoni"
 __copyright__ = """Copyright 2014, The Technical University of Denmark"""
