@@ -37,3 +37,18 @@ class bcolors:
         self.WARNING = ''
         self.FAIL = ''
         self.ENDC = ''
+
+
+def print_ok(string):
+    print bcolors.OKGREEN + "[SUCCESS] " + string + bcolors.ENDC
+
+def print_fail(string,msg=''):
+    print bcolors.FAIL + "[FAILED] " + string + bcolors.ENDC
+    if msg != '':
+        print bcolors.FAIL + msg + bcolors.ENDC
+
+def print_summary(title,nsucc,nfail):
+    if nfail > 0:
+        print bcolors.FAIL + "[SUMMARY " + title + "] Passed: " + str(nsucc) + "/" + str(nsucc+nfail) + bcolors.ENDC
+    else:
+        print bcolors.OKBLUE + "[SUMMARY " + title + "] Passed: " + str(nsucc) + "/" + str(nsucc+nfail) + bcolors.ENDC
